<?php

namespace Drupal\materio_migrate\Plugin\migrate\source;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\State\StateInterface;
use Drupal\migrate\Plugin\MigrationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\user_email_verification\UserEmailVerificationInterface;

/**
 * Drupal 7 user source from database.
 *
 * @MigrateSource(
 *   id = "d7_users",
 *   source_module = "user"
 * )
 */
class D7Users extends FieldableEntity {

  /**
   * User email verification helper service.
   *
   * @var \Drupal\user_email_verification\UserEmailVerificationInterface
   */
  protected $userEmailVerification;


  /**
   * Constructs a new UserEmailVerificationRequestForm object.
   *
   * @param \Drupal\user_email_verification\UserEmailVerificationInterface $user_email_verification_service
   *   User email verification helper service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current active user.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration, StateInterface $state, EntityManagerInterface $entity_manager, ModuleHandlerInterface $module_handler, UserEmailVerificationInterface $user_email_verification_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $state, $entity_manager);    
    $this->userEmailVerification = $user_email_verification_service;
  }


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('state'),
      $container->get('entity.manager'),
      $container->get('module_handler'),
      $container->get('user_email_verification.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    return $this->select('users', 'u')
      ->fields('u')
      ->condition('u.uid', 1, '>');
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      'uid' => $this->t('User ID'),
      'name' => $this->t('Username'),
      'pass' => $this->t('Password'),
      'mail' => $this->t('Email address'),
      'signature' => $this->t('Signature'),
      'signature_format' => $this->t('Signature format'),
      'created' => $this->t('Registered timestamp'),
      'access' => $this->t('Last access timestamp'),
      'login' => $this->t('Last login timestamp'),
      'status' => $this->t('Status'),
      'timezone' => $this->t('Timezone'),
      'language' => $this->t('Language'),
      'picture' => $this->t('Picture'),
      'init' => $this->t('Init'),
      'data' => $this->t('User data'),
      'roles' => $this->t('Roles'),
    ];

    // Profile fields.
    if ($this->moduleExists('profile')) {
      $fields += $this->select('profile_fields', 'pf')
        ->fields('pf', ['name', 'title'])
        ->execute()
        ->fetchAllKeyed();
    }

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    $uid = $row->getSourceProperty('uid');

    $roles = $this->select('users_roles', 'ur')
      ->fields('ur', ['rid'])
      ->condition('ur.uid', $uid)
      ->execute()
      ->fetchCol();
    $row->setSourceProperty('roles', $roles);

    $row->setSourceProperty('data', unserialize($row->getSourceProperty('data')));

    // If this entity was translated using Entity Translation, we need to get
    // its source language to get the field values in the right language.
    // The translations will be migrated by the d7_user_entity_translation
    // migration.
    $entity_translatable = $this->isEntityTranslatable('user');
    $source_language = $this->getEntityTranslationSourceLanguage('user', $uid);
    $language = $entity_translatable && $source_language ? $source_language : $row->getSourceProperty('language');
    $row->setSourceProperty('entity_language', $language);

    // Get Field API field values.
    foreach ($this->getFields('user') as $field_name => $field) {
      // Ensure we're using the right language if the entity and the field are
      // translatable.
      $field_language = $entity_translatable && $field['translatable'] ? $language : NULL;
      $field_values = $this->getFieldValues('user', $field_name, $uid, NULL, $field_language);
      // $field_values = [];
      // if ($field_name === 'field_company') {
      //   // print($uid . " ---\n");
      //   // print_r($tmp_field_values);
      //   // fix the taxo tid as tode module replace it by node nid
      //   // get the real tode tid
      //   foreach ($tmp_field_values as $key => $value) {
      //     $tode_tid = $this->select('field_data_field_tode_company', 'tode')
      //     ->condition('entity_type', 'node')
      //     ->condition('entity_id', $value['tid'])
      //     ->fields('tode', ['field_tode_company_tid'])
      //     ->execute()->fetchField();
      //     $field_values[] = array('tid'=>$tode_tid);
      //   }
      //   // print_r($field_values);
      //   $row->setSourceProperty('companies', $field_values);
      // }
      $row->setSourceProperty($field_name, $field_values);
    }

    // Get profile field values. This code is lifted directly from the D6
    // ProfileFieldValues plugin.
    if ($this->getDatabase()->schema()->tableExists('profile_value')) {
      $query = $this->select('profile_value', 'pv')
        ->fields('pv', ['fid', 'value']);
      $query->leftJoin('profile_field', 'pf', 'pf.fid=pv.fid');
      $query->fields('pf', ['name', 'type']);
      $query->condition('uid', $row->getSourceProperty('uid'));
      $results = $query->execute();

      foreach ($results as $profile_value) {
        if ($profile_value['type'] == 'date') {
          $date = unserialize($profile_value['value']);
          $date = date('Y-m-d', mktime(0, 0, 0, $date['month'], $date['day'], $date['year']));
          $row->setSourceProperty($profile_value['name'], ['value' => $date]);
        }
        elseif ($profile_value['type'] == 'list') {
          // Explode by newline and comma.
          $row->setSourceProperty($profile_value['name'], preg_split("/[\r\n,]+/", $profile_value['value']));
        }
        else {
          $row->setSourceProperty($profile_value['name'], [$profile_value['value']]);
        }
      }
    }

    // Member Type
    // check if aaf
    $field_memo = $row->getSourceProperty('field_memo');
    if ( stripos($field_memo[0]['value'], 'membre aaf') !== false ){
      // drush_print($row->getSourceProperty('mail').' member_type: AAF');
      $row->setSourceProperty('member_type', 2);
    } else {
      // get the uber_cart order infos
      $query = $this->select('uc_orders', 'uco')
        ->fields('uco', ['uid', 'order_id']);
      $query->leftJoin('uc_order_products', 'ucop', 'ucop.order_id=uco.order_id');
      $query->fields('ucop', ['model']);
      $query->condition('uid', $row->getSourceProperty('uid'));
      $query->condition('ucop.nid', 11849);
      $query->orderBy('uco.created', 'DESC');
      $orders = $query->execute()->fetchAll();
      
      
      if(count($orders)){
        $first_order = array_shift($orders);
        // drush_print($row->getSourceProperty('mail'));
        // drush_print(print_r($first_order, true));
        switch($first_order['model']){
          case "abo01-A":
            $member_type = 0;
            break;
          case "abo01-B":
            $member_type = 1;
            break;
          case "abo01-C":
            $member_type = 1;
            // add bad phone number to memo field
            $memo .= "#migration: multijoueurs \n";
            break;
        }
        if(isset($member_type)){
          // drush_print($row->getSourceProperty('mail').' member_type: '.$member_type);
          $row->setSourceProperty('member_type', $member_type);
        }

      }
      // abo01-A 1 (web)
      // abo01-B 2 (web+showroom)
      // abo01-C 2 (web+showroom)
    }
    
    // set the user's email ferified
    $this->userEmailVerification->setEmailVerifiedByUserId($row->getSourceProperty('uid'));

    // record migration errors in field_memo
    if(isset($memo)){
      $field_memo = $row->getSourceProperty('field_memo');
      $field_memo[0]['value'] .= "\n".$memo;
      $row->setSourceProperty('field_memo', $field_memo);
    }

    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'uid' => [
        'type' => 'integer',
        'alias' => 'u',
      ],
    ];
  }

}
